#!/usr/bin/env python3
import pandas
import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import brokenaxes
import re

BOX_WIDTH = 4
BOX_HEIGHT = 6
DATA_WIDTH = 0.6

# Helper functions
# Strips URLs to get an ID
def strip_urls(base, url):
    return int(url.replace(base, ""))

# Load CSV file and aggregate by mean time
def load_frame(filename, base_url):
    frame = pandas.read_csv(filename)
    frame['id'] = frame['id'].apply(lambda x: strip_urls(base_url, x))
    return frame.groupby(['id'], as_index = False).mean()

# Joins two frames together, renaming columns as necessary
def join_frames(links_frame, java_frame, col):
    joined_frame = pandas.merge(left=links_frame,\
                                right=java_frame, left_on='id', right_on='id')
    links_col = col + '_x'
    java_col = col + '_y'
    joined_frame =\
        joined_frame.rename(columns={ links_col: "Links",\
                                      java_col: "Java"})
    return joined_frame

def box_plot(joined_frame, output_filename, title, y_label, ax_break_start, ax_break_end):
    jf = joined_frame[['Links', 'Java']]
    fig, (ax1, ax2) = plt.subplots(2, 1,gridspec_kw={'height_ratios': [0.5, 2]}, sharex=True)
    jf.boxplot(ax=ax1, widths=DATA_WIDTH)
    ax1.set_xlabel('')
    ax1.set_ylim(ymin=ax_break_end)

    jf.boxplot(ax=ax2, widths=DATA_WIDTH)
    ax2.set_title('')
    ax2.set_ylim(ymin=0, ymax=ax_break_start)
    fig.set_size_inches(BOX_WIDTH, BOX_HEIGHT)
    fig.subplots_adjust(top=0.75)

    ax1.spines['bottom'].set_visible(False)
    ax2.spines['top'].set_visible(False)
    ax1.xaxis.tick_top()
    ax1.tick_params(labeltop=False)  # don't put tick labels at the top
    ax2.xaxis.tick_bottom()

    d = .015  # how big to make the diagonal lines in axes coordinates
    d4 = d * 4
    # arguments to pass to plot, just so we don't keep repeating them
    kwargs = dict(transform=ax1.transAxes, color='k', clip_on=False)
    ax1.plot((-d, +d), (-d, +d), **kwargs)        # top-left diagonal
    ax1.plot((1 - d, 1 + d), (-d, +d), **kwargs)  # top-right diagonal

    kwargs.update(transform=ax2.transAxes)  # switch to the bottom axes
    ax2.plot((-d, +d), (1 - d, 1 + d), **kwargs)  # bottom-left diagonal
    ax2.plot((1 - d, 1 + d), (1 - d, 1 + d), **kwargs)  # bottom-right diagonal

    plt.ylabel(y_label)
    ax1.title.set_text(title)
    fig.savefig(output_filename, bbox_inches='tight')


def box_plot_nobreak(joined_frame, output_filename, title, y_label):
    jf = joined_frame[['Links', 'Java']]
    fig, ax1 = plt.subplots() # plt.subplots(2, 1,gridspec_kw={'height_ratios': [0.5, 2]}, sharex=True)
    jf.boxplot(ax=ax1, widths=DATA_WIDTH)
    ax1.set_xlabel('')
    fig.set_size_inches(BOX_WIDTH, BOX_HEIGHT)
    fig.subplots_adjust(top=0.75)
    plt.ylabel(y_label)
    ax1.title.set_text(title)
    fig.savefig(output_filename, bbox_inches='tight')


def object_query_count(links_frame, java_frame):
    # Drop the execution time columns from both frames
    links_frame = links_frame.drop(['execution_time', 'total_time'], axis=1)
    java_frame = java_frame.drop(['total_time'], axis=1)
    joined_frame = join_frames(links_frame, java_frame, 'query_count')
    box_plot(joined_frame, 'object_querycount.pdf', 'Query count (Object Data)',\
             'Query count', 1500, 2000)

def object_query_time(links_frame, java_frame):
    # Drop the execution time columns from both frames
    links_frame = links_frame.drop(['execution_time', 'query_count'], axis=1)
    java_frame = java_frame.drop(['query_count'], axis=1)
    joined_frame = join_frames(links_frame, java_frame, 'total_time')
    box_plot(joined_frame, 'object_querytime.pdf', 'Query time (Object Data)',\
             'Query time (ms)', 400, 600)

   #box_plot_nobreak(joined_frame, 'object_querytime.pdf', 'Query time (Object Data)',\
   #                 'Query time (ms)')

def object_page_time(links_frames, java_frames):
    # Need to generate two plots: first is including query time, second is without
    (links_query, links_time) = links_frames
    (java_query, java_time) = java_frames

    # First bit is easy: just do what we did before
    joined_time = join_frames(links_time, java_time, 'time')
    box_plot(joined_time, 'object_pagetime.pdf', 'Page time (Object Data)',\
             'Page build time (ms)', 1250, 2000)

    # Now, need to subtract query time
    links_query = links_query.drop(['execution_time', 'query_count'], axis=1)
    java_query = java_query.drop(['query_count'], axis=1)
    joined_query = join_frames(links_query, java_query, 'total_time')
    # Now, join with the time frame in order to subtract the query time
    joined = pandas.merge(left=joined_time, right=joined_query, left_on='id', right_on='id')
    links_time = joined.apply(lambda x: x['Links_x'] - x['Links_y'], axis=1)
    java_time = joined.apply(lambda x: x['Java_x'] - x['Java_y'], axis=1)
    joined = joined.drop(['Links_x', 'Links_y', 'Java_x', 'Java_y'], axis=1)
    joined.insert(1, 'Links', links_time)
    joined.insert(1, 'Java', java_time)
    joined = joined.set_index('id')
    box_plot(joined, 'object_pagetime_noquery.pdf', 'Page time excl. query time (Object Data)',\
             'Page build time (ms)', 1250, 1800)


def object_page():
    # Load the frames
    links_query_frame = \
        load_frame('../results/links_objectdisplay_querytime.csv',\
                   '//localhost:8080/objectDisplay?objectId=')
    java_query_frame =\
        load_frame('../results/java_objectdisplay_querytime.csv',\
                   'http://www.guidetopharmacology.org:8090/GRAC/ObjectDisplayForward?objectId=')
    # Load the frames
    links_time_frame = \
        load_frame('../results/links_objectdisplay_pagetime.csv',\
                   '//localhost:8080/objectDisplay?objectId=')
    java_time_frame =\
        load_frame('../results/java_objectdisplay_pagetime.csv',\
                   'http://www.guidetopharmacology.org:8090/GRAC/ObjectDisplayForward?objectId=')
    # Generate the charts
    object_query_count(links_query_frame, java_query_frame)
    object_query_time(links_query_frame, java_query_frame)
    object_page_time((links_query_frame, links_time_frame),\
                     (java_query_frame, java_time_frame))


## List logic is slightly different
def strip_list_urls(url, base_url):
    x = re.sub(base_url, '', url)
    x = re.sub('type=', '', x)
    return re.sub('null', '', x)

def transform_list_frame(frame, base_url):
    frame['id'] = frame['id'].apply(lambda x: strip_list_urls(x, base_url))
    return frame.groupby(['id']).mean()


def generate_bar_chart(filename, figsize, frame, spacing, ticks):
    plt.figure(figsize=figsize)
    num_items = len(ticks)
    pos = np.arange(num_items)

    links_query_times = frame['links_query_time']
    links_times = frame['links_page_time'] - frame['links_query_time']
    java_query_times = frame['java_query_time']
    java_times = frame['java_page_time'] - frame['java_query_time']

    bar_width = (1.0 - (2.0 * spacing)) / num_items
    p1 = plt.bar(pos, links_query_times, width=bar_width, color='#EE5334')
    p2 = plt.bar(pos, links_times, width=bar_width, bottom=links_query_times, color='#0F0C59')
    p3 = plt.bar(pos + bar_width, java_query_times, width=bar_width, color='#2A85C2')
    p4 = plt.bar(pos + bar_width, java_times, width=bar_width, bottom=java_query_times, color='#D98282')

    plt.ylabel('Time (ms)')
    plt.xticks(pos + bar_width / 2, tuple(ticks), rotation='horizontal')

    plt.title('Execution times for disease list')
    plt.legend((p1[0], p2[0], p3[0], p4[0]), ('Query Time (Links)', 'Page Build Time (Links)', 'Query Time (Java)', 'Page Build Time (Java)'))
    plt.savefig(filename, bbox_inches='tight')

def generate_bar_chart_break(filename, figsize, frame, ticks, break_start, break_end):
    # Generate plots and set limits
    (fig, (ax1, ax2)) = plt.subplots(2, 1, gridspec_kw={'height_ratios': [0.4, 1]}, sharex=True, figsize=figsize)
    ax2.set_ylim(0, break_start)
    ax1.set_ylim(ymin = break_end, ymax=32000)
    ax1.spines['bottom'].set_visible(False)
    ax1.tick_params(axis='x',which='both',bottom=False)
    ax2.spines['top'].set_visible(False)
    num_items = len(ticks)
    pos = np.arange(num_items)

    # Plot stufff

    links_query_times = frame['links_query_time']
    links_times = frame['links_page_time'] - frame['links_query_time']
    java_query_times = frame['java_query_time']
    java_times = frame['java_page_time'] - frame['java_query_time']

    bar_width = 0.4
    # Plot on the first subchart
    p1 = ax2.bar(pos, links_query_times, width=bar_width, color='#EE5334')
    p2 = ax2.bar(pos, links_times, width=bar_width, bottom=links_query_times, color='#0F0C59')
    p3 = ax2.bar(pos + bar_width, java_query_times, width=bar_width, color='#2A85C2')
    p4 = ax2.bar(pos + bar_width, java_times, width=bar_width, bottom=java_query_times, color='#D98282')
    # Now on the second
    p5 = ax1.bar(pos, links_query_times, width=bar_width, color='#EE5334')
    p6 = ax1.bar(pos, links_times, width=bar_width, bottom=links_query_times, color='#0F0C59')
    p7 = ax1.bar(pos + bar_width, java_query_times, width=bar_width, color='#2A85C2')
    p8 = ax1.bar(pos + bar_width, java_times, width=bar_width, bottom=java_query_times, color='#D98282')

    # Add break ticks
    for tick in ax2.get_xticklabels():
        tick.set_rotation(0)
    d = .01
    scale = 2
    kwargs = dict(transform=ax1.transAxes, color='k', clip_on=False)
    ax1.plot((-d, +d), (-d, +d), **kwargs)
    ax1.plot((1 - d, 1 + d), (-d, +d), **kwargs)
    kwargs.update(transform=ax2.transAxes)
    ax2.plot((-d, +d), (1 - d, 1 + d), **kwargs)
    ax2.plot((1 - d, 1 + d), (1 - d, 1 + d), **kwargs)

    bars1 = [p1, p2, p3, p4]
    bars2 = [p5, p6, p7, p8]

    for c1, c2 in zip(bars1, bars2):
        for b in c1.get_children():
            posx = b.get_x() + b.get_width()/2.
            if b.get_height() > break_start:
                ax2.plot((posx-3*d, posx+3*d), (1 - d, 1 + d), color='k', clip_on=False,
                         transform=ax2.get_xaxis_transform())
        for b in c2.get_children():
            posx = b.get_x() + b.get_width()/2.
            if b.get_height() > break_end:
                ax1.plot((posx-3*d, posx+3*d), (- d, + d), color='k', clip_on=False,
                         transform=ax1.get_xaxis_transform())
    # Labels
    ax2.set_ylabel('Time (ms)')
    ax2.set_xticks(pos + bar_width / 2)#, tuple(ticks), rotation='horizontal')
    ax2.set_xticklabels(ticks, fontsize=8, rotation='horizontal')
    ax1.set_title('Execution times for ligand list')
    ax1.legend((p1[0], p2[0], p3[0], p4[0]), ('Query Time (Links)', 'Page Build Time (Links)', 'Query Time (Java)', 'Page Build Time (Java)'))
    plt.savefig(filename, bbox_inches='tight')

def disease_list():
    # Load CSVs
    links_disease_query = transform_list_frame(pandas.read_csv('../results/links_diseaselist_querytime.csv'),
                                         '//localhost:8080/diseaseList\??')
    links_disease_page = transform_list_frame(pandas.read_csv('../results/links_diseaselist_pagetime.csv'),
                                         '//localhost:8080/diseaseList\??')

    java_disease_query = transform_list_frame(pandas.read_csv('../results/java_diseaselist_querytime.csv'),
                                         'http://www.guidetopharmacology.org:8090/GRAC/DiseaseListForward\??')
    java_disease_page = transform_list_frame(pandas.read_csv('../results/java_diseaselist_pagetime.csv'),
                                         'http://www.guidetopharmacology.org:8090/GRAC/DiseaseListForward\??')

    # Join query frames
    disease_query_joined =\
    pandas.merge(left=links_disease_query, right=java_disease_query, left_on='id', right_on='id')\
    .rename(columns = {"query_count_x": "links_query_count",\
                       "query_count_y": "java_query_count",
                       "execution_time": "links_execution_time",
                       "total_time_x": "links_query_time",
                       "total_time_y": "java_query_time"
                          })

    # Strip irrelevant fields
    disease_query_joined = disease_query_joined.drop(['links_query_count', 'java_query_count',
                                                      'links_execution_time'], axis=1)

    # Join page time frames
    disease_page_joined =\
        pandas.merge(left=links_disease_page, right=java_disease_page, left_on='id', right_on='id')\
        .rename(columns = {"time_x": "links_page_time",
                           "time_y": "java_page_time"
                          })
    # Finally, join the query and time frames
    disease_joined =\
        pandas.merge(left=disease_query_joined, right=disease_page_joined, left_on='id', right_on='id')

    print(disease_joined)
    def tick_lbl(x):
        if x == '':
            return 'All'
        else:
            return x

    ticks = [ tick_lbl(x) for x in disease_joined.index]
    generate_bar_chart('disease_list.pdf', (4,9), disease_joined, 0.15, ticks)

def ligand_list():
    # Load CSVs
    links_ligand_query = transform_list_frame(pandas.read_csv('../results/links_ligandlist_querytime.csv'),
                                         '//localhost:8080/ligandList\??')
    links_ligand_page = transform_list_frame(pandas.read_csv('../results/links_ligandlist_pagetime.csv'),
                                         '//localhost:8080/ligandList\??')

    java_ligand_query = transform_list_frame(pandas.read_csv('../results/java_ligandlist_querytime.csv'),
                                         'http://www.guidetopharmacology.org:8090/GRAC/LigandListForward\?database=all&type=')
    java_ligand_page = transform_list_frame(pandas.read_csv('../results/java_ligandlist_pagetime.csv'),
                                         'http://www.guidetopharmacology.org:8090/GRAC/LigandListForward\?database=all&type=')

    # Join query frames
    ligand_query_joined =\
    pandas.merge(left=links_ligand_query, right=java_ligand_query, left_on='id', right_on='id')\
    .rename(columns = {"query_count_x": "links_query_count",\
                       "query_count_y": "java_query_count",
                       "execution_time": "links_execution_time",
                       "total_time_x": "links_query_time",
                       "total_time_y": "java_query_time"
                          })
    print("LQJ: " + str(ligand_query_joined))

    # Strip irrelevant fields
    ligand_query_joined = ligand_query_joined.drop(['links_query_count', 'java_query_count',
                                                      'links_execution_time'], axis=1)

    # Join page time frames
    ligand_page_joined =\
        pandas.merge(left=links_ligand_page, right=java_ligand_page, left_on='id', right_on='id')\
        .rename(columns = {"time_x": "links_page_time",
                           "time_y": "java_page_time"
                          })
    # Finally, join the query and time frames
    ligand_joined =\
        pandas.merge(left=ligand_query_joined, right=ligand_page_joined, left_on='id', right_on='id')

    print(ligand_joined)
    def tick_lbl(x):
        if x == '':
            return 'Approved'
        elif x == 'Endogenous-peptide':
            return 'Endog.\nPeptide'
        elif x == 'Natural-product':
            return 'Nat.\nProduct'
        elif x == 'Synthetic-organic':
            return 'Synth.\nOrganic'
        elif x == 'WHO-essential':
            return 'WHO\nessential'
        else:
            return x

    ticks = [ tick_lbl(x) for x in ligand_joined.index]
    generate_bar_chart_break('ligand_list.pdf', (10,6), ligand_joined, ticks, 14000, 29000)


def disease_query_count(links_frame, java_frame):
    # Drop the execution time columns from both frames
    links_frame = links_frame.drop(['execution_time', 'total_time'], axis=1)
    java_frame = java_frame.drop(['total_time'], axis=1)
    joined_frame = join_frames(links_frame, java_frame, 'query_count')
    box_plot(joined_frame, 'disease_querycount.pdf', 'Query count (Disease Data)',\
             'Query count', 320, 1300)

def disease_query_time(links_frame, java_frame):
    # Drop the execution time columns from both frames
    links_frame = links_frame.drop(['execution_time', 'query_count'], axis=1)
    java_frame = java_frame.drop(['query_count'], axis=1)
    joined_frame = join_frames(links_frame, java_frame, 'total_time')
    box_plot(joined_frame, 'disease_querytime.pdf', 'Query time (Disease Data)',\
             'Query time (ms)', 120, 300)

def disease_page_time(links_frames, java_frames):
    # Need to generate two plots: first is including query time, second is without
    (links_query, links_time) = links_frames
    (java_query, java_time) = java_frames

    # First bit is easy: just do what we did before
    joined_time = join_frames(links_time, java_time, 'time')
    box_plot(joined_time, 'disease_pagetime.pdf', 'Page time (Disease Data)',\
             'Page build time (ms)', 1600, 2200)

    # Now, need to subtract query time
    links_query = links_query.drop(['execution_time', 'query_count'], axis=1)
    java_query = java_query.drop(['query_count'], axis=1)
    joined_query = join_frames(links_query, java_query, 'total_time')
    # Now, join with the time frame in order to subtract the query time
    joined = pandas.merge(left=joined_time, right=joined_query, left_on='id', right_on='id')
    links_time = joined.apply(lambda x: x['Links_x'] - x['Links_y'], axis=1)
    java_time = joined.apply(lambda x: x['Java_x'] - x['Java_y'], axis=1)
    joined = joined.drop(['Links_x', 'Links_y', 'Java_x', 'Java_y'], axis=1)
    joined.insert(1, 'Links', links_time)
    joined.insert(1, 'Java', java_time)
    joined = joined.set_index('id')
    box_plot(joined, 'disease_pagetime_noquery.pdf', 'Page time excl. query time (Disease Data)',\
             'Page build time (ms)', 1600, 2000)

def disease_page():
    # Load the frames
    links_query_frame = \
        load_frame('../results/links_diseasedisplay_querytime.csv',\
                   '//localhost:8080/diseaseDisplay?diseaseId=')
    java_query_frame =\
        load_frame('../results/java_diseasedisplay_querytime.csv',\
                   'http://www.guidetopharmacology.org:8090/GRAC/DiseaseDisplayForward?diseaseId=')
    # Load the frames
    links_time_frame = \
        load_frame('../results/links_diseasedisplay_pagetime.csv',\
                   '//localhost:8080/diseaseDisplay?diseaseId=')
    java_time_frame =\
        load_frame('../results/java_diseasedisplay_pagetime.csv',\
                   'http://www.guidetopharmacology.org:8090/GRAC/DiseaseDisplayForward?diseaseId=')
    # Generate the charts
    disease_query_count(links_query_frame, java_query_frame)
    disease_query_time(links_query_frame, java_query_frame)
    disease_page_time((links_query_frame, links_time_frame),\
                     (java_query_frame, java_time_frame))

def main():
    object_page()
    disease_page()
    disease_list()
    ligand_list()

if __name__ == '__main__':
    main()
