#!/usr/bin/env python3
from io import StringIO
import pandas
import numpy as np
import brokenaxes

# Generates summary statistics (median, std dev, min, max) for the query time
# and build times for each page


PAGES = ['diseasedisplay', 'diseaselist', 'familydisplay',\
         'liganddisplay', 'ligandfamilies', 'ligandlist', \
         'objectdisplay', 'receptorfamilies']

def links_query_filename(x):
    return '../results/links_' + x + '_querytime.csv'

def links_page_filename(x):
    return '../results/links_' + x + '_pagetime.csv'

def java_query_filename(x):
    return '../results/java_' + x + '_querytime.csv'

def java_page_filename(x):
    return '../results/java_' + x + '_pagetime.csv'

def load_frame(filename):
    return pandas.read_csv(filename)

def summarise_by(col, frame):
    def summarise(x):
        return { 'mean': x.mean(), 'median': x.median(),\
                 'std_dev': x.std(), 'min': x.min(), 'max': x.max() }
    return summarise(frame[col])


def summarise_page_frame(links_frame, java_frame):
    # Group by ID, get the mean of the repeats
    grouped_links = links_frame.groupby(['id'], as_index = False).mean()
    grouped_java = java_frame.groupby(['id'], as_index = False).mean()

    # Slice by 'time'
    links_obj = summarise_by('time', grouped_links)
    java_obj = summarise_by('time', grouped_java)
    return { 'time': { 'links': links_obj, 'java': java_obj } }

def summarise_query_frame(links_frame, java_frame):
    # Group by ID, get the mean of the repeats
    grouped_links = links_frame.groupby(['id'], as_index = False).mean()
    grouped_java = java_frame.groupby(['id'], as_index = False).mean()

    links_count = summarise_by('query_count', grouped_links)
    java_count = summarise_by('query_count', grouped_java)
    links_time = summarise_by('total_time', grouped_links)
    java_time = summarise_by('total_time', grouped_java)

    return { 'count': { 'links': links_count, 'java': java_count },\
             'time': { 'links': links_time, 'java': java_time } }

def print_latex_page_table(rows):
    for (pg_name, data) in rows:
        l = data['time']['links']
        j = data['time']['java']
        print("%s & %.2f & %.2f & %.2f & %.2f & %.2f & %.2f & %.2f & %.2f \\\\ \\hline" %\
              (pg_name, l['median'], l['std_dev'], l['min'], l['max'], \
               j['median'], j['std_dev'], j['min'], j['max'] ))


def print_latex_query_tables(rows):
    def print_tbl(dimension):
        print(dimension)
        for (pg_name, data) in rows:
            l = data[dimension]['links']
            j = data[dimension]['java']
            print("%s & %.2f & %.2f & %.2f & %.2f & %.2f & %.2f & %.2f & %.2f \\\\ \\hline" %\
                  (pg_name, l['median'], l['std_dev'], l['min'], l['max'], \
                   j['median'], j['std_dev'], j['min'], j['max'] ))

    print_tbl('count')
    print_tbl('time')


def main():

    def page_row(page):
        links_frame = load_frame(links_page_filename(page))
        java_frame = load_frame(java_page_filename(page))
        return (page, summarise_page_frame(links_frame, java_frame))

    def query_row(page):
        links_frame = load_frame(links_query_filename(page))
        java_frame = load_frame(java_query_filename(page))
        return (page, summarise_query_frame(links_frame, java_frame))

    page_rows = [ page_row(page) for page in PAGES ]
    query_rows = [ query_row(page) for page in PAGES ]
    print_latex_page_table(page_rows)
    print_latex_query_tables(query_rows)

if __name__ == '__main__':
    main()
